import compileall
import os
import runpy

from shutil import copy as scopy
from os import remove
from . import MyPath

def copy(src, dst):
    pdst = os.path.dirname(dst)
    if not os.path.exists(pdst):  # 如果路径不存在
        os.makedirs(pdst)
    scopy(src, dst)

def delete_gap_dir(dir):
        if os.path.isdir(dir):
            for d in os.listdir(dir):
                delete_gap_dir(os.path.join(dir, d))
            if not os.listdir(dir):
                os.rmdir(dir)

def compile_to(
        src=r'package',
        dst=r'release_folder',
        py_version='36', name='EasyPro',
        version=None,
        delete=False,
        upload=True,
):
    src = MyPath(src)
    dst = MyPath(dst)
    if version is None:
        from datetime import datetime
        version = py_version+'.'+datetime.now().strftime("%Y.%m.%d.%H.%M")
    version = version.replace('.0','.')


    release_folder = dst.cat('release_'+name)
    pakage_copy = release_folder.cat(name)

    # region clear
    for root, dirs, files in os.walk(src):
        for file in files:
            if '.pyc' in file:
                path = MyPath(root).cat(file)
                remove(path)

    for root, dirs, files in os.walk(pakage_copy):
        for file in files:
            path = MyPath(root).cat(file)
            remove(path)

    delete_gap_dir(pakage_copy)
    # endregion

    for root, dirs, files in os.walk(src):
        for file in files:
            src_path = MyPath(root).cat(file)
            dst_path = src_path.replace(src, pakage_copy)
            if '.py' in file and not'.pyc' in file:
                dst_path = dst_path.replace('/__pycache__', '').replace('.cpython-' + py_version, '')
                copy(src_path, dst_path)
                continue

    # generate setup
    ds = pakage_copy.split('/')
    dst_path = MyPath('').cat(*ds[:-1])

    f = dst_path.cat('setup.py')
    f.ensure()
    f = open(f, 'w')
    f.write(
        """# -*- coding:utf-8 -*-
import sys
sys.argv.append('sdist')
from distutils.core import setup
from setuptools import find_packages

setup(name=\'""" + name + """\',
            version=\'""" + version + """\',
            packages=find_packages(),  
            description='A python lib for xxxxx',
            long_description='',
            author='Quanfa',
            package_data={
            '': ['*.py'],
            },
            author_email='quanfa@tju.edu.cn',
            url='http://www.xxxxx.com/',
            license='MIT',
            )

            """
    )
    f.close()
    if True:
        f = dst_path.cat('compile.py')
        f.ensure()
        f = open(f, 'w')
        f.write(
            """# -*- coding:utf-8 -*-
from os.path import abspath
path = abspath(__file__).replace('\\\\', '/')
mask = 'T20'
paths = path.split('/')
root_path = ''
for c in paths:
    root_path += c
    if mask in c:
        break
    root_path += '/'
import sys

sys.path.append(root_path)
from distutils.core import setup
from setuptools import find_packages
import os
sys.argv.append('sdist')
dst_path = \'"""+dst_path+ """\'
os.chdir(dst_path)
import runpy
runpy.run_path(r\'"""+dst_path.cat('setup.py')+"""\')
                    """
        )
        f.close()

        runpy.run_path(dst_path.cat('compile.py'))

        release_name = name + '-' + version + '.tar.gz'
        rp = dst.cat(release_name)
        copy(dst_path.cat('dist', release_name), rp)

    if delete:
        for root, dirs, files in os.walk(release_folder):
            for file in files:
                path = os.path.join(root, file)
                remove(path)

        try:
            delete_gap_dir(release_folder)
        except:
            pass


    if upload:
        mycmd = f"python -m twine upload  -u quanfa -p Py@70539 {rp}"
        s = os.system(mycmd)
        print(s)


def compile_(
        src: MyPath=r'package',
        dst: MyPath=r'release_folder',
        py_version='36', name='EasyPro',
        version=None,
        delete=False,
):
    if version is None:
        from datetime import datetime
        version = py_version+'.'+datetime.now().strftime("%Y.%m.%d.%H.%M")
    version = version.replace('.0','.')

    release_folder = dst.cat('release_'+name)
    pakage_copy = release_folder.cat(name)

    for root, dirs, files in os.walk(src):
        for file in files:
            if '.pyc' in file:
                path = MyPath(root).cat(file)
                remove(path)

    for root, dirs, files in os.walk(pakage_copy):
        for file in files:
            path = MyPath(root).cat(file)
            remove(path)
    def delete_gap_dir(dir):
        if os.path.isdir(dir):
            for d in os.listdir(dir):
                delete_gap_dir(os.path.join(dir, d))
            if not os.listdir(dir):
                os.rmdir(dir)

    delete_gap_dir(pakage_copy)

    compileall.compile_dir(src)

    for root, dirs, files in os.walk(src):
        for file in files:
            src_path = MyPath(root).cat(file)
            dst_path = src_path.replace(src, pakage_copy)
            if '.pyc' in file:
                if '__init__' in file:
                    continue
                dst_path = dst_path.replace('/__pycache__', '').replace('.cpython-' + py_version, '')
                copy(src_path, dst_path)
                continue
            if '__init__.md' in file:
                copy(src_path, dst_path)

    # generate setup
    ds = pakage_copy.split('/')
    dst_path = MyPath('').cat(*ds[:-1])

    f = dst_path.cat('setup.py')
    f.ensure()
    f = open(f, 'w')
    f.write(
        """# -*- coding:utf-8 -*-
import sys
sys.argv.append('sdist')
from distutils.core import setup
from setuptools import find_packages

setup(name=\'""" + name + """\',
            version=\'""" + version + """\',
            packages=find_packages(),  
            description='A python lib for xxxxx',
            long_description='',
            author='Quanfa',
            package_data={
            '': ['*.pyc'],
            },
            author_email='quanfa@tju.edu.cn',
            url='http://www.xxxxx.com/',
            license='MIT',
            )

            """
    )
    f.close()
    if True:
        f = dst_path.cat('compile.py')
        f.ensure()
        f = open(f, 'w')
        f.write(
            """# -*- coding:utf-8 -*-
from os.path import abspath
path = abspath(__file__).replace('\\\\', '/')
mask = 'T20'
paths = path.split('/')
root_path = ''
for c in paths:
    root_path += c
    if mask in c:
        break
    root_path += '/'
import sys

sys.path.append(root_path)
from distutils.core import setup
from setuptools import find_packages
import os
sys.argv.append('sdist')
dst_path = \'"""+dst_path+ """\'
os.chdir(dst_path)
import runpy
runpy.run_path(r\'"""+dst_path.cat('setup.py')+"""\')
                    """
        )
        f.close()

        runpy.run_path(dst_path.cat('compile.py'))

        release_name = name + '-' + version + '.tar.gz'

        release_tar = dst.cat(release_name)

        copy(dst_path.cat('dist', release_name), release_tar)

    if delete:
        for root, dirs, files in os.walk(release_folder):
            for file in files:
                path = os.path.join(root, file)
                remove(path)

        try:
            delete_gap_dir(release_folder)
        except:
            pass





from .path_tool import MyPath


class Project(MyPath):
    # region basic structure

    structure = [
        'mylocals',
        'myscripts',
        'myreadmes',

        'myreadmes/catalog.md',
        'myscripts/__init__.py',
        '.gitignore'
    ]

    @property
    def locals_path(self):
        return MyPath(self.cat(self.structure[0]))

    @property
    def scripts_path(self):
        return MyPath(self.cat(self.structure[1]))

    @property
    def readmes_path(self):
        return MyPath(self.cat(self.structure[2]))

    # endregion

    def check_project_path(self, next=False):
        name = self.get_name()
        if not 'T20' in name:
            print(self, '真的是项目根目录吗？')
            return next
        return True

    def check_project_structure(self, next=False):
        r = True
        if not self.check_project_path(next):
            return next

        for structure in self.structure:
            path = self.cat(structure)
            if not path.exist():
                r = False
                print('文件(夹)', structure, '不存在')

        if not r:
            return next
        return True

    # region create structure

    def create_structure(self):

        for structure in self.structure:
            path = self.cat(structure)
            if not path.exist():
                path.ensure()
                print('文件(夹)', structure, '已创建')

        with open(self.cat(self.structure[3]), 'w') as f:
            f.write("""# abstract

# authors
"""
                    )

        with open(self.cat(self.structure[4]), 'w') as f:
            f.write("""
from EasyPro import ScriptFileSaver, MyPath

sfs = ScriptFileSaver(__file__, locals())

p0 = 'master'

# endregion

if __name__ == '__main__':
    from EasyPro import Project

    project = Project(MyPath.from_file(__file__).get_parent().get_parent())
    project.create_script(p0, 'test')

            """)
        with open(self.cat(self.structure[5]), 'w') as f:
            f.write("""
.idea
mylocals
            """)
        self.add_author('master')
        self.create_script('master', 'test')

    def add_author(self, author):
        author_script_path = self.scripts_path.cat(author)
        if author_script_path.exist():
            print('author exist')
            return
        author_script_path.ensure()
        author_script_path.cat('__init__.py').ensure()

        author_md_folder = self.readmes_path.cat(author).ensure()
        author_md = author_md_folder.cat(f'{author}.md').ensure()
        with open(author_md, 'w') as f:
            f.write("""# abstract


# from
[catalog](..%2Fcatalog.md)

# catalog
"""
                    )

        catalog_md = self.readmes_path.cat('catalog.md')
        with open(catalog_md, 'a') as f:
            f.write(f'\n\n[{author}]({author}%2F{author}.md)')

    def create_script(self, author_name, script_name):

        if not self.scripts_path.cat(author_name).exist():
            self.add_author(author_name)

        script_index = len(self.scripts_path.cat(author_name).get_files(list_r=True))
        script_name = 's' + str(script_index) + '_' + script_name

        script_path = self.scripts_path.cat(author_name).cat(script_name + '.py')
        markdown_path = self.readmes_path.cat(author_name).cat(script_name + '.md')

        if script_path.exist():
            print('script in this name has already existed in your scripts:', script_path,
                  'Try to use git if you want to edit your script to a new branch.')
            return False

        script_path.ensure()
        print('create script at', script_path)
        with open(script_path, 'w') as f:
            f.write(
                """# -*- coding: utf-8 -*-
from EasyPro import ScriptFileSaver
sfs = ScriptFileSaver(__file__, locals())

if __name__ == '__main__':
    print('Running has finished')
         """
            )

        markdown_path.ensure()
        print('create markdown at', markdown_path)
        with open(markdown_path, 'w') as f:
            f.write(f"""# [{script_name}.py](..%2F..%2Fmyscripts%2F{author_name}%2F{script_name}.py)
    

# from
[{author_name}]({author_name}.md)
    
    """
                    )

        catalog_md = self.readmes_path.cat(author_name).cat(f'{author_name}.md')
        with open(catalog_md, 'a') as f:
            f.write(f'\n\n[{script_name}]({script_name}.md)')

    # endregion

    # region create
    @classmethod
    def create_at(cls, projects_path, project_name):
        from datetime import datetime
        date = datetime.now().strftime("%Y%m%d")
        project_name = 'T' + date + '_' + project_name
        projects_path = MyPath(projects_path)
        project_path = projects_path.cat(project_name)
        project = cls(project_path)
        project.create_structure()
    # endregion



# -*- coding: utf-8 -*-
# @Time    : 2023/1/12 12:03
# @Author  : Quanfa
# @Desc    :

from .path_tool import MyPath
from .matlab import save_mat, load_mat
from .project import Project
import torch as saver
import sys


def save(object, path, name, suffix):
    if suffix == 'Figure':
        object.savefig(path)
    if suffix == 'mat':
        save_mat(object, path, name)
    else:
        saver.save(object, path)
    print('save ', name, ' at ', path)


class ScriptFileSaver:
    def __init__(self, script_file, locals, version: int = None):
        """
        A combination of database and saver in framework.

        :param root_path: local path
        :param date_mark:
        :param version:
        :param author:
        """

        if version is None:
            version = 1
        self._locals = locals

        # region calculate version
        script_path = MyPath.from_file(script_file)
        project_path = Project(script_path.get_level(0, -3))
        author_name = script_path.get_level(-2)
        script_name = script_path.get_name()[:-3]
        if script_path.get_parent() == project_path.scripts_path:  # 如果是脚本生成脚本
            project_path = Project(script_path.get_level(0, -2))
            author_name = 'auto'
            script_name = 'script_create'

        save_path_parent = project_path.locals_path.cat(author_name).cat(script_name).ensure()

        # endregion
        self._local_path = save_path_parent.cat('s' + str(version))
        self._version = version
        self._local_path.ensure()
        self._root_path = project_path

        # region append project path to system
        if not project_path in sys.path:
            sys.path.append(project_path)
        # endregion

    def path_of(self, file_name='auto_save_result', suffix='sus'):
        """

        :param file_name:
        :return:
        """
        if suffix == '':
            path = self._local_path.cat(file_name)
        else:
            path = self._local_path.cat(file_name + '.' + suffix)

        return path

    def save(self, name, object=None, suffix=None, path=None):
        """
        保存变量，任意类型的python对象
        :param name: 保存的名字
        :param object: 如果没给定，就自动从内存中搜索
        :param suffix: sus, sci util saved; mat, matlab
        :return:
        """
        if object is None:
            object = self._locals[name]
        if suffix is None:
            suffix = str(type(object)).split("'")[1].split('.')[-1]
        if path is None:
            path = self.path_of(name, suffix)
        else:
            path = MyPath(path)

        save(object, path, name, suffix)
        return path

    def load(self, name=None, suffix=None, object_sample=None, path=None):
        """
        load from specified version.
        :param name:
        :return:
        """
        if path is None:
            if suffix is None:
                path = self._local_path.get_files(mark=name, list_r=True)[0]
                suffix = path.split('.')[-1]
            else:
                path = self.path_of(name, suffix)
        print('load ', suffix, ' from ', path)
        if object_sample is not None:
            return object_sample.load(path)
        if suffix == 'mat':
            return load_mat(path)
        else:
            return saver.load(path)

    def __getitem__(self, name):
        return self.load(path=self.path_of(name, ''))



# -*- coding: utf-8 -*-
# @Time    : 2023/1/12 13:06
# @Author  : Quanfa
# @Desc    : 
# region import
from torch import Tensor
# endregion

def save_mat(object, path, name) -> str:
    import numpy as np
    from scipy import io
    object = np.array(object)
    object = np.squeeze(object)
    mdic = {name: object}
    io.savemat(path, mdic)

    return path


def load_mat(path) -> Tensor:
    from scipy import io
    result = io.loadmat(path)
    r = Tensor(result[list(result.keys())[3]])
    return r

